"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = default_1;
const schematics_1 = require("@angular-devkit/schematics");
const xplat_1 = require("@nstudio/xplat");
const xplat_utils_1 = require("@nstudio/xplat-utils");
const utils_1 = require("../../utils");
const electron_1 = require("@nstudio/electron");
const devkit_1 = require("@nx/devkit");
function default_1(options) {
    if (!options.name) {
        throw new schematics_1.SchematicsException((0, xplat_1.missingArgument)('name', 'Provide a name for your Electron app.', 'nx g @nstudio/electron-angular:app name'));
    }
    if (!options.target) {
        throw new schematics_1.SchematicsException((0, xplat_1.missingArgument)('target', 'Provide the name of the web app in your workspace to use inside the electron app.', 'nx g @nstudio/electron-angular:app name --target web-myapp'));
    }
    return (0, schematics_1.chain)([
        (0, xplat_utils_1.prerun)(options),
        xplat_1.XplatHelpers.applyAppNamingConvention(options, 'electron'),
        (tree, context) => (0, schematics_1.externalSchematic)('@nstudio/electron', 'xplat', Object.assign(Object.assign({}, options), { skipDependentPlatformFiles: true })),
        // use xplat or not
        options.useXplat
            ? (0, schematics_1.externalSchematic)('@nstudio/electron-angular', 'xplat', options)
            : (0, schematics_1.noop)(),
        (tree, context) => addAppFiles(options, options.name)(tree, context),
        utils_1.XplatElectronAngularHelpers.updateRootDeps(options),
        electron_1.XplatElectrontHelpers.addNpmScripts(options),
        (tree, context) => {
            const fullTargetAppName = options.target;
            let targetConfig;
            try {
                targetConfig = (0, devkit_1.readProjectConfiguration)(tree, fullTargetAppName);
            }
            catch (e) { }
            if (!targetConfig) {
                throw new schematics_1.SchematicsException(`The target app name "${fullTargetAppName}" does not appear to be in the workspace. You may need to generate the app first or perhaps check the spelling.`);
            }
            const electronAppName = options.name;
            if (!targetConfig.root) {
                targetConfig.root = `apps/${electronAppName}`;
            }
            let targetProp = 'architect';
            if (!targetConfig[targetProp]) {
                targetProp = 'targets'; // nx 11 moved to 'targets'
            }
            if (targetConfig[targetProp]) {
                // update to use electron module
                targetConfig[targetProp].build.options.outputPath = `dist/apps/${electronAppName}`;
                if (options.useXplat) {
                    targetConfig[targetProp].build.options.main = `apps/${fullTargetAppName}/src/main.electron.ts`;
                    targetConfig[targetProp].build.options.tsConfig = `apps/${fullTargetAppName}/tsconfig.electron.json`;
                }
                targetConfig[targetProp].build.options.assets.push({
                    glob: '**/*',
                    input: `apps/${electronAppName}/src/`,
                    ignore: ['**/*.ts'],
                    output: '',
                });
                if (targetConfig[targetProp].serve.options) {
                    targetConfig[targetProp].serve.options.browserTarget = `${electronAppName}:build`;
                }
                if (targetConfig[targetProp].serve.configurations.production) {
                    targetConfig[targetProp].serve.configurations.production.browserTarget = `${electronAppName}:build:production`;
                }
                if (targetConfig[targetProp].serve.configurations.development) {
                    targetConfig[targetProp].serve.configurations.development.browserTarget = `${electronAppName}:build:development`;
                }
                // clear other settings (TODO: may need these in future), for now keep electron options minimal
                delete targetConfig[targetProp]['extract-i18n'];
                delete targetConfig[targetProp]['test'];
                delete targetConfig[targetProp]['lint'];
            }
            return (0, devkit_1.updateProjectConfiguration)(tree, fullTargetAppName, Object.assign({ name: electronAppName }, targetConfig));
        },
        options.useXplat ? (tree) => adjustAppFiles(options, tree) : (0, schematics_1.noop)(),
        electron_1.XplatElectrontHelpers.noteAppCommands(options),
    ]);
}
function addAppFiles(options, appPath) {
    const appname = (0, xplat_utils_1.getAppName)(options, 'electron');
    const directory = options.directory ? `${options.directory}/` : '';
    return (0, schematics_1.branchAndMerge)((0, schematics_1.mergeWith)((0, schematics_1.apply)((0, schematics_1.url)(`./_files`), [
        (0, schematics_1.template)(Object.assign(Object.assign(Object.assign({}, options), (0, xplat_1.getDefaultTemplateOptions)()), { appname, xplatFolderName: xplat_1.XplatHelpers.getXplatFoldername('electron', 'angular') })),
        (0, schematics_1.move)(`apps/${directory}${appPath}`),
    ])));
}
function adjustAppFiles(options, tree) {
    const fullTargetAppName = options.target;
    const electronModulePath = `/apps/${fullTargetAppName}/src/app/app.electron.module.ts`;
    if (!tree.exists(electronModulePath)) {
        tree.create(electronModulePath, electronModule());
    }
    const electronMainPath = `/apps/${fullTargetAppName}/src/main.electron.ts`;
    if (!tree.exists(electronMainPath)) {
        tree.create(electronMainPath, electronMain());
    }
    const electronTsConfigPath = `/apps/${fullTargetAppName}/tsconfig.electron.json`;
    if (!tree.exists(electronTsConfigPath)) {
        tree.create(electronTsConfigPath, electronTsConfig());
    }
    return tree;
}
function electronModule() {
    return `import { NgModule } from '@angular/core';
import { ${xplat_1.stringUtils.classify((0, xplat_utils_1.getPrefix)())}ElectronCoreModule } from '@${(0, xplat_utils_1.getNpmScope)()}/xplat/${xplat_1.XplatHelpers.getXplatFoldername('electron', 'angular')}/core';
import { AppModule } from './app.module';
import { AppComponent } from './app.component';

@NgModule({
  imports: [AppModule, ${xplat_1.stringUtils.classify((0, xplat_utils_1.getPrefix)())}ElectronCoreModule],
  bootstrap: [AppComponent]
})
export class AppElectronModule {}`;
}
function electronMain() {
    return `import { enableProdMode } from '@angular/core';
  import { platformBrowserDynamic } from '@angular/platform-browser-dynamic';
  
  // libs
  import { environment } from '@${(0, xplat_utils_1.getNpmScope)()}/xplat/core';
  
  // app
  import { AppElectronModule } from './app/app.electron.module';
  
  if (environment.production) {
    enableProdMode();
  }
  
  platformBrowserDynamic()
    .bootstrapModule(AppElectronModule)
    .catch(err => console.log(err));
  `;
}
function electronTsConfig() {
    return `{
  "extends": "./tsconfig.json",
  "compilerOptions": {
    "outDir": "../../dist/out-tsc",
    "types": []
  },
  "files": ["src/main.electron.ts", "src/polyfills.ts"],
  "include": ["src/**/*.d.ts"],
  "exclude": ["src/test-setup.ts", "**/*.spec.ts"]
}`;
}
