"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const schematics_1 = require("@angular-devkit/schematics");
const xplat_utils_1 = require("@nstudio/xplat-utils");
const testing_1 = require("@nstudio/xplat/testing");
const testing_2 = require("../../utils/testing");
describe('app', () => {
    let appTree;
    const defaultOptions = {
        name: 'foo',
        target: 'web-viewer',
        npmScope: 'testing',
        prefix: 'tt',
        isTesting: true,
    };
    beforeEach(() => {
        appTree = schematics_1.Tree.empty();
        appTree = (0, testing_1.createXplatWithAppsForElectron)(appTree);
    });
    it('should create all files of an app', () => __awaiter(void 0, void 0, void 0, function* () {
        const options = Object.assign({}, defaultOptions);
        // console.log('appTree:', appTree);
        const tree = yield (0, testing_2.runSchematic)('app', options, appTree);
        const files = tree.files;
        // console.log(files);
        let checkPath = '/apps/electron-foo/src/index.ts';
        expect(files.indexOf(checkPath)).toBeGreaterThanOrEqual(0);
        let checkFile = (0, testing_1.getFileContent)(tree, checkPath);
        expect(checkFile.indexOf(`path.join(__dirname, 'index.html')`)).toBeGreaterThanOrEqual(0);
        expect(files.indexOf('/apps/electron-foo/src/icons/icon.png')).toBeGreaterThanOrEqual(0);
        expect(files.indexOf('/apps/electron-foo/tsconfig.json')).toBeGreaterThanOrEqual(0);
        checkPath = '/apps/electron-foo/src/package.json';
        expect(files.indexOf(checkPath)).toBeGreaterThanOrEqual(0);
        checkFile = (0, testing_1.getFileContent)(tree, checkPath);
        // console.log(checkFile);
        expect(checkFile.indexOf(`"name": "foo"`)).toBeGreaterThanOrEqual(0);
        checkPath = '/package.json';
        expect(files.indexOf(checkPath)).toBeGreaterThanOrEqual(0);
        checkFile = (0, testing_1.getFileContent)(tree, checkPath);
        // console.log(checkFile);
        const packageData = (0, xplat_utils_1.jsonParse)(checkFile);
        expect(packageData.scripts['postinstall']).toBeDefined();
        expect(packageData.scripts['build.electron.foo']).toBeDefined();
        expect(packageData.scripts['build.electron.foo.local']).toBeDefined();
        expect(packageData.scripts['build.electron.foo.linux']).toBeDefined();
        expect(packageData.scripts['build.electron.foo.windows']).toBeDefined();
        expect(packageData.scripts['build.electron.foo.mac']).toBeDefined();
        expect(packageData.scripts['prepare.electron.foo']).toBeDefined();
        expect(packageData.scripts['serve.electron.foo.target']).toBeDefined();
        expect(packageData.scripts['serve.electron.foo']).toBeDefined();
        expect(packageData.scripts['start.electron.foo']).toBeDefined();
        // check target web app for supporting files
        checkPath = '/apps/web-viewer/src/app/app.electron.module.ts';
        expect(files.indexOf(checkPath)).toBeGreaterThanOrEqual(0);
        checkFile = (0, testing_1.getFileContent)(tree, checkPath);
        // console.log(checkFile);
        expect(checkFile.indexOf(`TtElectronCoreModule`)).toBeGreaterThanOrEqual(0);
        expect(checkFile.indexOf(`AppElectronModule`)).toBeGreaterThanOrEqual(0);
        checkPath = '/apps/web-viewer/src/main.electron.ts';
        expect(files.indexOf(checkPath)).toBeGreaterThanOrEqual(0);
        checkFile = (0, testing_1.getFileContent)(tree, checkPath);
        // console.log(checkFile);
        expect(checkFile.indexOf(`./app/app.electron.module`)).toBeGreaterThanOrEqual(0);
        checkPath = '/apps/web-viewer/tsconfig.electron.json';
        expect(files.indexOf(checkPath)).toBeGreaterThanOrEqual(0);
        // make sure start script is correct
        checkPath = '/package.json';
        checkFile = (0, testing_1.getFileContent)(tree, checkPath);
        // console.log(checkFile);
        expect(checkFile.indexOf(`npm run prepare.electron.${options.name} && npm-run-all -p serve.electron.${options.name}.target serve.electron.${options.name}`)).toBeGreaterThanOrEqual(0);
    }));
    it('should create all files for app in directory', () => __awaiter(void 0, void 0, void 0, function* () {
        const options = Object.assign({}, defaultOptions);
        options.directory = 'frontend';
        const tree = yield (0, testing_2.runSchematic)('app', options, appTree);
        // const files = tree.files;
        // console.log(files);
        expect(tree.exists('/apps/frontend/electron-foo/src/index.ts')).toBeTruthy();
    }));
    it('should create all files for app in directory and ignore platform naming when directory is a platform', () => __awaiter(void 0, void 0, void 0, function* () {
        const options = Object.assign({}, defaultOptions);
        options.directory = 'electron';
        const tree = yield (0, testing_2.runSchematic)('app', options, appTree);
        // const files = tree.files;
        // console.log(files);
        expect(tree.exists('/apps/electron/foo/src/index.ts')).toBeTruthy();
    }));
    describe('useXplat false', () => {
        it('should geneate app with no connections to xplat architecture', () => __awaiter(void 0, void 0, void 0, function* () {
            appTree = schematics_1.Tree.empty();
            appTree.create('/apps/web-viewer/package.json', JSON.stringify({}));
            appTree.create('angular.json', JSON.stringify({
                version: 1,
                projects: {
                    'web-viewer': {
                        targets: {
                            build: {
                                options: {
                                    assets: [],
                                },
                            },
                            serve: {
                                options: {},
                                configurations: {
                                    production: {},
                                },
                            },
                        },
                    },
                },
            }));
            appTree.create('/package.json', JSON.stringify({
                dependencies: {},
                devDependencies: {},
                xplat: {},
            }));
            appTree.create('/nx.json', JSON.stringify({ npmScope: 'testing', projects: {} }));
            appTree.create('/tsconfig.json', JSON.stringify({ compilerOptions: { paths: {} } }));
            const options = Object.assign({}, defaultOptions);
            options.useXplat = false;
            // console.log('appTree:', appTree);
            const tree = yield (0, testing_2.runSchematic)('app', options, appTree);
            // const files = tree.files;
            // console.log(files);
            // should *not* create this
            expect(tree.exists('/apps/web-viewer/src/main.electron.ts')).toBeFalsy();
        }));
    });
});
